function [D, dtype, nrows, ncols, bo, N] = estparse( fname )
% Function ESTPARSE( fname );
%
% Function which interprets the header of EST_Track files
%
% [D] = ESTPARSE( fname )
%
% This simplest form just returns D, the file offset where data starts 
%
% [D, dtype, nrows, ncols, bo, N] = ESTPARSE( fname )
% 
% Optionally returns the correct ByteOrder (bo) string necessary when 
% opening the file, and the format (ascii/binary), as well as the 
% number of rows (nrows) and number of columns (ncols) and N, a cell
% array of channel names.
%
% See also ESTLOAD

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%    Author: Korin Richmond                                    %%
%%   Purpose: Function for parsing EST headers                  %%
%%   Created: Fri Feb 11 16:10:04 2000                          %%
%%                                                              %%
%%  Copyright (C) Korin Richmond 2000                           %%
%%                                                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                              %%
%% Licence conditions apply to the download and use of this     %%
%% software - please see the file "LICENCE" accompanying this   %%
%% file in the same directory for details                       %%
%%                                                              %%
%% DISCLAIMER:                                                  %%
%%                                                              %%
%% EDINBURGH UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK       %%
%% DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,        %%
%% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND      %%
%% FITNESS. IN NO EVENT SHALL EDINBURGH UNIVERSITY OR THE       %%
%% CONTRIBUTORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR          %%
%% CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING    %%
%% FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF   %%
%% CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT   %%
%% OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS      %% 
%% SOFTWARE.                                                    %%
%%                                                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[fid, msg] = fopen( fname, 'r', 'l' );

if fid < 2 
  errmsg=sprintf( 'Could not open file %s\n', fname );
  error( errmsg );
end

line = fgets( fid );
if ~strncmp( line, 'EST_File Track', 14 )
  error( ['File ' fname ' does not appear to be EST_Track file'] );
end

D=0; % what the hell - assume no header
bo='n'; % assume Native byte order...

while ~strcmp( line, 'EST_Header_End' )
  line = fgets( fid );
  line(end)=[]; 
  switch line
    case 'DataType ascii'
      dtype='ascii';
    case 'DataType binary'
      dtype='binary';
    case 'ByteOrder 01'
      bo='l';
    case 'ByteOrder 10'
      bo='b';
    otherwise
      if strncmp( line, 'NumFrames ', 10 )
        nrows=sscanf( line, 'NumFrames %d', 1 );
      elseif strncmp( line, 'NumChannels ', 12 )
        ncols=sscanf( line, 'NumChannels %d', 1 );
        N=cell(ncols, 1);
      elseif strncmp( line, 'Channel_', 8 )
        m = regexpi( line, 'Channel_(?<chn>\d+)\s+(?<name>\S+)', 'names' );
        N{str2num(m.chn)+1}=m.name;
      end
  end
end

D=ftell(fid);

fclose(fid);
